package com.oracle.ticketsystem.dao.tests;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.oracle.ticketsystem.beans.Ticket;
import com.oracle.ticketsystem.dao.ITicketDao;
import com.oracle.ticketsystem.dao.impl.TicketJPADao;

public class TicketDaoTest {

	
	private ITicketDao ticketDao = null;
	private static long testTicketId = 0;

	@Before
	public void setUp() throws Exception {
		ticketDao = new TicketJPADao();
	}

	@Test
	public void testAdd() {
		
		// adding a test ticket
		Ticket ticket = ticketDao.add(1001, "Mark", "mark@gmail.com", "TestTicket", "Ticket about the test");
		
		assertTrue("Could not add test ticket.", (ticket.getId() > 0));
		
		testTicketId = ticket.getId();
		
	}

	@Test
	public void testGet() {
		
		// finding a test ticket
		Ticket ticket = ticketDao.get(testTicketId);
		
		assertNotNull("Could not find test ticket.", ticket);
		
	}

	@Test
	public void testUpdate() {
		
		// finding a test ticket
		Ticket ticket = ticketDao.update(testTicketId, "peter", "Getting the test ticket", "ASSIGNED");
		
		assertNotNull("Could not update the test ticket.", ticket.getTechnician());
		assertTrue("Could not add history to test ticket.", (ticket.getTicketHistory().size() > 0));
		
	}

	@Test
	public void testGetTicketsOwnedByTechnician() {
		
		// finding a test ticket
		List<Ticket> tickets = ticketDao.getTicketsOwnedByTechnician("peter");
		
		assertTrue("Could not find tickets assigned to technician 'peter'", (tickets.size() > 0));
		
	}

	@Test
	public void testGetOpenTickets() {
		
		// finding a test ticket
		List<Ticket> tickets = ticketDao.getOpenTickets();
		
		assertTrue("Could not find open tickets.", (tickets.size() > 0));
		
	}

	@Test
	public void testRemove() {
		
		// finding a test ticket
		ticketDao.remove(testTicketId);
		
		// the ticket has been removed
		assertTrue(true);
		
	}
	
}
