package com.oracle.ticketsystem.dao.impl;

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import com.oracle.ticketsystem.beans.Product;
import com.oracle.ticketsystem.beans.Technician;
import com.oracle.ticketsystem.beans.Ticket;
import com.oracle.ticketsystem.beans.Tickethistory;
import com.oracle.ticketsystem.dao.ITicketDao;

public class TicketJPADao extends BaseJPADao implements ITicketDao {

	/**
	 * Date pattern used across the application.
	 */
	public static final String DATE_PATTERN = "MM/dd/yyyy hh:mm:ss aa"; //$NON-NLS-1$

	/**
	 * Default no-arg constructor
	 */
	public TicketJPADao() {
	}
	
	@Override
	public Ticket add(long productId, 
			String customerName, String customerEmail, 
			String title, String description) {
		
		Product product = getEntityManager().find(Product.class, productId);
		
		if(product == null) {
			throw new RuntimeException("While adding a new ticket, " +
					"could not find reference to the given product Id: " + productId);
		}
		
		
		Ticket ticket = new Ticket();
		ticket.setProduct(product);
		ticket.setCustomername(customerName);
		ticket.setCustomeremail(customerEmail);
		ticket.setTitle(title);
		ticket.setDescription(description);
		ticket.setState("NEW"); // always NEW state
		SimpleDateFormat dtFormat = new SimpleDateFormat(DATE_PATTERN);
		ticket.setSubmissiondate(dtFormat.format(new Date()));
		
		Long maxId = getMaxId("SELECT max(t.id) FROM Ticket t");
		
		// setting the ticket Id
		ticket.setId( (maxId == null) ? 0 : maxId + 1);
		
		EntityTransaction t = getEntityManager().getTransaction();
		t.begin();
		getEntityManager().persist(ticket);
		t.commit();
		
		return ticket;
	}

	@Override
	public Ticket get(long ticketId) {
		return getEntityManager().find(Ticket.class, ticketId);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Ticket> getTicketsOwnedByTechnician(String technicianId) {
		
		Query query = getEntityManager().createQuery("SELECT t from Ticket t WHERE t.technician.id = :technicianId");
		query.setParameter("technicianId", technicianId);
		
		return query.getResultList();
	}

	@Override
	public Ticket update(long ticketId, String technicianId, String comment,
			String state) {
		
		EntityTransaction t = getEntityManager().getTransaction();
		t.begin();
		
		Ticket ticket = get(ticketId);
		
		if(ticket == null) {
			return null;
		}
		
		ticket.setState(state);
		Technician technician = null;
		
		if(technicianId != null) {
			
			technician = getEntityManager().find(Technician.class, technicianId);
			
			if(technician == null) {
				throw new RuntimeException("No technician found for the ID '" + technicianId + "'");
			}
			
		}
		ticket.setTechnician(technician);
		
		
		
		Tickethistory ticketHistory = new Tickethistory();
		Long maxTicketHistoryId = getMaxId("SELECT max(h.id) FROM Tickethistory h");
		
		// setting the ticketHistory Id
		ticketHistory.setId( (maxTicketHistoryId == null) ? 0 : maxTicketHistoryId + 1);
		
		if(technician != null) {
			ticketHistory.setTechnician(technician);
		}
		ticketHistory.setState(state);
		ticketHistory.setComments(comment);
		ticketHistory.setTicket(ticket);
		
		SimpleDateFormat dtFormat = new SimpleDateFormat(DATE_PATTERN);
		ticketHistory.setUpdatedate(dtFormat.format(new Date()));
		
		getEntityManager().persist(ticketHistory);
		
		ticket.getTicketHistory().add(ticketHistory);
		
		t.commit();
		return ticket;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public List<Ticket> getOpenTickets() {
		
		Query openTicketsQuery = getEntityManager().createQuery(
							"SELECT t FROM Ticket t " +
							"WHERE t.state = :newState " +
							"OR t.state = :openState");
		openTicketsQuery.setParameter("newState", "NEW");
		openTicketsQuery.setParameter("openState", "OPEN");
		
		return openTicketsQuery.getResultList();
	}

	@Override
	public void remove(long ticketId) {
		
		Ticket ticket = get(ticketId);
		if(ticket != null) {
			EntityTransaction trx = getEntityManager().getTransaction();
			trx.begin();
			
			for(Tickethistory ticketHistory : ticket.getTicketHistory()) {
				getEntityManager().remove(ticketHistory);
			}
			
			ticket.getTicketHistory().clear();
			
			getEntityManager().remove(ticket);
			
			trx.commit();
		}
		
	}
	
	private long getMaxId(String maxQuery) {
		
		Query maxIdQuery = getEntityManager().createQuery(maxQuery);
		
		Long maxId = 1L;
		
		if( (maxIdQuery.getResultList() != null) && (maxIdQuery.getResultList().size() > 0) ) {
			maxId = (Long)maxIdQuery.getResultList().get(0);
		}
		return maxId;
	}
	
}
